/*
 *  Requires:
 *      incorporateurk.js
 *      utils.js
 */

// Initalize psiturk object
var psiTurk = new PsiTurk(uniqueId, adServerLoc, mode);

/********************
 * Task Parameters
 *
 * Sets task parameters like number of lists, list length, presentation rate,
 *
 ********************/

// user determined task params
var num_of_lists = 3; // number of word lists to study
var list_length = 10; // number of words in each word list
var pres_rate = 2000; // number of milleseconds each word presented for
var isi = 1000; // number of ms of blank screen between word presentations
var recall_time = 60000; // number of milliseconds given to recall
var delay_between_lists = 5000; // number of milliseconds to pause between lists (display get ready message)
var recall_box_lag = 0; // number of ms to ignore input into the text box after recall period starts --- so people don't accidently enter responses to the math task here
var word_pool; // function in utils.js

// fixed task params

// temp divert everyone into distinctive condition
var task_condition = 0;

// preallocate and initialize variables
var cur_list_num = 0; //counter to keep track of current list --- uses zero indexing

/********************
* HTML manipulation
*
* All HTML files in the templates directory are requested 
* from the server when the PsiTurk object is created above. We
* need code to get those pages from the PsiTurk object and 
* insert them into the document.
*
********************/

// pick task page (second instruction page) based on task_condition
if (task_condition==0) {
    word_pool = make_control_pool();
}
else if (task_condition==1) {
    word_pool = make_distinct_pool();
}

var instructionPages = ["instructions/instructions.html", "instructions/instructions-ready.html"];

// List of Task html pages
var pages = [
    "stage.html",
    "postquestionnaire.html",
    "debriefing.html"
];

// load all pages we will use
psiTurk.preloadPages(instructionPages);
psiTurk.preloadPages(pages);

/*********************
 * Free recall task  *
 *********************/
var RunFR = function() {

    /*********************************
     * Setup variables for this list *
     ********************************/
    var cur_phase,
        wordon, // time word is presented
        listening = false,
        first_recall = true, // keep tack of whether this is the first recall for a list
        stims = word_pool.splice(0,list_length); // get the items for this list: the next list_length elements of word pool


    /*************************************************************************************
     * Function to advance to the next item or recall depending on the phase of the list *
     *************************************************************************************/

    var next = function() {

        //figure out which phase of the task we are in

        // what to do if recall phase
        if (stims.length===0) { // if there are no stims left, we have entered the recall phase
            if (first_recall) {
                remove_word()
                cur_phase = "RECALL"
                start_time = new Date().getTime();
            }
            recall_period()
        }

        // what to do if study phase
        else { // otherwise we still have items to present

            cur_phase = "STUDY"
            stim = stims.shift();

            psiTurk.recordTrialData({
                    'task_condition': task_condition,
                    'list': cur_list_num,
                    'phase': "study",
                    'word': stim[0],
                }
            );
            psiTurk.saveData();

            if (stims.length===list_length-1) {
                if (cur_list_num==0) {
                    ready_message = "The first list will begin shortly."
                }
                else {
                    ready_message = "A new list will begin shortly."
                }
                d3.select("#stim")
                    .append("div")
                    .attr("id","word")
                    .style("color","black")
                    .style("text-align","center")
                    .style("font-size","40px")
                    .style("font-weight","400")
                    .style("margin","300px 0px 0px 0px")
                    .text(ready_message);
                setTimeout(function(){present_fixation(isi, stim[0], pres_rate); }, delay_between_lists);

            }
            else{
                present_item(stim[0], pres_rate);
                }

        }

    };

    /******
     * Function to record responses as appropriate for the phase (study vs recall)
     *
     * This control the flow of the experiment. It keeps track of how many words have been presented and starts the
     * recall period when all the words are done
     *
     * Then it keeps track of the number of milliseconds the recall period has been going on and ends when time is up
     *
     * Then it checks how many lists have been done and runs another if there are more left.
     *
     * This is all handled through recursion rather than for loops
     *
     ****/

    var response_handler = function(e) {

        var keyCode = e.keyCode,
            subjects_answer,
            recalled_item,
            response;

        // handler for the recall phase
        if (cur_phase === "RECALL") {

            // only accept input if the recall text box is in focus
            box_in_focus = document.activeElement.name == "recall_field";
            if (!box_in_focus)  {
                d3.select("#recall_input").html('<span  style="color: red;">Click inside textbox to activate it before typing!:</span> ' +
                    '<input type="text" id="recall_field" name="recall_field"/>');
                    return
            }

            switch (keyCode) {
                case 13:
                    recalled_item = document.getElementById("recall_field").value
                    break;
            }

            if (recalled_item.length > 0) {

                listening = false;

                var elapsed = new Date().getTime() - start_time;
                psiTurk.recordTrialData({
                        'task_condition': task_condition,
                        'list': cur_list_num,
                        'phase': "recall",
                        'response': recalled_item,
                        'rt': elapsed
                    }
                );
                psiTurk.saveData();
                next();

            }

        }

    };


    /****************************************************************
     * Function to end the FR task once all the lists are presented *
     ****************************************************************/
    var finish = function() {
        $("body").unbind("keydown", response_handler); // Unbind keys
        currentview = new Questionnaire();
    };

    /********************************
     * Function to present fixation *
     ********************************/

    var present_fixation = function(fix_duration, text, duration) {

        // remove any words already on screen
        remove_word()

        // show the word for pres_rate ms
        d3.select("#task").html("<p> </p>");
        d3.select("#query").html('<p id="prompt"> </p>');
        d3.select("#stim")
            .append("div")
            .attr("id","word")
            .style("color","black")
            .style("text-align","center")
            .style("font-size","50px")
            .style("font-weight","400")
            .style("margin","300px 0px 0px 0px")
            .text('+');

        // start listening and record start time
        wordon = new Date().getTime();
        setTimeout(function(){present_item(text, duration); }, fix_duration);

    };

    /*******************************
     * Function to present an item *
     *******************************/

    var present_item = function(text, duration) {

        // remove any words already on screen
        remove_word()

        // show the word for pres_rate ms
        d3.select("#task").html("<p> </p>");
        d3.select("#query").html('<p id="prompt"> </p>');
        d3.select("#stim")
            .append("div")
            .attr("id","word")
            .style("color","black")
            .style("text-align","center")
            .style("font-size","50px")
            .style("font-weight","400")
            .style("margin","300px 0px 0px 0px")
            .text(text);

        // start listening and record start time
        wordon = new Date().getTime();
        setTimeout(function(){wrapup_word(); }, duration);

    };

     /***************************************************
     * Function to present text box for recalling words *
     ****************************************************/

    var recall_period = function() {
        //remove text from encoding task
        d3.select("#query").remove();

        // display input box
        disp_this = '<p>You now have ' + recall_time/1000 +
            ' seconds to to try and recall the words from the list you just saw. ' +
            'You can recall the words in any order. Try to recall as many words as you can. If you cannot remember any more words, that is okay; the task will automatically advance when the time is up.</p>'
        d3.select("#task").html(disp_this);
        d3.select("#recall_input").html('<span>Type a word and press ENTER to submit:</span> ' +
            '<input type="text" id="recall_field" name="recall_field"/>');

        // want to ensure people don't accidently try to enter their last response from the math distractor here
        // so defocus the textbox if this is the first recall to avoid too rapid a response
        if (!first_recall) {
            d3.select("#recall_field").node().focus()
        }

        // start listening
        listening = true;

        if (first_recall) {
            document.body.style.backgroundColor = "white";
            setTimeout(function(){wrapup_recall(); }, recall_time);
            setTimeout(function(){d3.select("#recall_field").node().focus(); }, recall_box_lag/2);
            first_recall = false
        }

    };

    /******************************************************************************************
     * Function to record a time out response if no response was made to the word and move on *
     ******************************************************************************************/

     var wrapup_word = function() {

        // show a fixation for isi ms
        // remove text from encoding task
        // stop listening

        listening = false;

        d3.select("#word").remove();
        d3.select("#stim")
            .append("div")
            .attr("id","word")
            .style("color","black")
            .style("text-align","center")
            .style("font-size","50px")
            .style("font-weight","400")
            .style("margin","300px 0px 0px 0px")
            .text("+");

        setTimeout(function(){next(); }, isi);

     }

     /********************************
     * Function to end recall period *
     *********************************/

     var wrapup_recall = function() {

        listening = false;

        var last_list = cur_list_num+1==num_of_lists; // check if we already have presented all the lists

        if (last_list) {
            finish()
        }

        else {
           cur_list_num++
           cur_phase = "STUDY"
           //psiturk.saveData()
           RunFR()
        }
     }

    /**********************************************
     * Function to remove an item from the screen *
     *********************************************/

    var remove_word = function() {
        d3.select("#word").remove();
    };

    /******************
     * Start the task *
     *****************/

    // Load the stage.html snippet into the body of the page
    psiTurk.showPage('stage.html');

    // Register the response handler that is defined above to handle any
    // key down events.
    $("body").focus().keydown(response_handler);

    // Start the test
    next();

};

/****************
* Questionnaire *
****************/

var Questionnaire = function() {

	record_responses = function() {

        // record strategy questions:
		$('input').each( function(i, val) {
		    psiTurk.recordTrialData({
                        'task_condition': task_condition,
                        'phase': "postquestionnaire",
                        'strategy': this.name,
                        'strategy_description': this.value,
                        'used': this.checked,
                        });
            psiTurk.saveData();
		});

	};

	// Load the questionnaire snippet
	psiTurk.showPage('postquestionnaire.html');
	psiTurk.recordTrialData({'phase':'postquestionnaire', 'status':'begin'});

	/* If you want to use a debriefing form:
	$("#next").click(function () {
	    record_responses();
	    psiTurk.saveData();
	    currentview = new Debrief();
	});
	*/

	$("#next").click(function () {
	    record_responses();
        psiTurk.computeBonus('compute_bonus', function() {
            psiTurk.completeHIT(); // when finished saving compute bonus, the quit
        });
	});

};

/******************
* Debriefing Form *
*******************/

var Debrief = function() {

	var error_message = "<h1>Oops!</h1><p>Something went wrong submitting your HIT. This might happen if you lose your internet connection. Press the button to resubmit.</p><button id='resubmit'>Resubmit</button>";

	prompt_resubmit = function() {
		document.body.innerHTML = error_message;
		$("#resubmit").click(resubmit);
	};

	resubmit = function() {
		document.body.innerHTML = "<h1>Trying to resubmit...</h1>";
		reprompt = setTimeout(prompt_resubmit, 10000);

		psiTurk.saveData({
			success: function() {
			    clearInterval(reprompt);
                psiTurk.computeBonus('compute_bonus', function(){finish()});
			},
			error: prompt_resubmit
		});
	};

	// Load the debriefing
	psiTurk.showPage('debriefing.html');
	psiTurk.recordTrialData({'phase':'debriefing', 'status':'begin'});
	$("#next").click(function () {
	    psiTurk.saveData({
            success: function(){
                psiTurk.computeBonus('compute_bonus', function() {
                	psiTurk.completeHIT(); // when finished saving compute bonus, the quit
                });
            },
            error: prompt_resubmit});
	});

};

/************
 * Run Task *
 ************/

// Task object to keep track of the current phase
var currentview;

$(window).load( function(){
    psiTurk.doInstructions(
    	instructionPages, // a list of pages you want to display in sequence
    	function() { currentview = new RunFR(); } // what you want to do when you are done with instructions
    );
});